module Planeten_Modul
! längste Berechung:

! DIM=3
! AUSGEBEN_ALLE=1
! DELTA_T=0.00001
! T_MAX=1000
! Planeten: 4 (ohne Sonne)
! ZEIT:   19m45.201s
!Platte voll

! Allgemeine Parameter
integer,parameter::DIM=3                 ! Raumdimension (2D oder 3D)
integer,parameter::AUSGEBEN_ALLE=50
real,parameter::   DELTA_T=0.01           ! Zeitschrittweite 
real,parameter::   T_MAX=3000             ! Endzeit
! Physikalische Konstanten
real,parameter::   GAMMA=1.177e-2        ! Gravitationskonstante GAMMA

! Eigene Datentypen
type Atom                                ! Himmelskoerper:
  real::m                                !   Masse
  real,dimension(DIM)::x,v,F,Falt             !   Ort,Geschwindigkeit,Kraft
  integer::typ                           !   Typ (fuer Darstellung)
end type
type Atome                               ! Sonnensystem:
  integer::N                             !   Anzahl der Himmelskoerper
  type(Atom),dimension(:),pointer::a     !   Feld der Himmelskoerper
end type

contains

! ***  Zeitintegration und Ausgabe  *******************************************

subroutine Ausgabe(A,t)
      type(Atome)::A
      real::t
  integer::i
  write(unit=1,fmt=*) A%N
  write(unit=1,fmt=*) t," fs"
  do i=1,A%N
    write(unit=1,fmt=*) A%a(i)%typ, A%a(i)%x, 0, 0
  end do
end subroutine 

subroutine Kraft(A,i,j)
      type(Atome)::A; integer::i,j
  real,dimension(DIM)::xij
  real::r,rr,Fij
  integer::d
  xij=A%a(j)%x-A%a(i)%x                               ! Abstandsvektor von i nach j
  rr=0.0                                              ! r^2
  do d=1,DIM
    rr=rr+xij(d)**2
  end do
  r=sqrt(rr)                                         ! Abstand
  Fij=GAMMA*A%a(i)%m*A%a(j)%m/rr                     ! Kraft von j auf i
  A%a(i)%F=A%a(i)%F+Fij*xij/r
end subroutine

subroutine Berechne_F(A)
      type(Atome)::A
   integer::i
   integer::j
   do i=1,A%N
     A%a(i)%F=0
   end do
   do i=1,A%N
    do j=1,A%N
      if (j/=i) then
       call Kraft(A,i,j)
      end if
    end do
   end do
end subroutine

subroutine Berechne_x(A,dt)
      type(Atome)::A; real::dt
  integer::i
  do i=1,A%N
    A%a(i)%x=A%a(i)%x+dt*A%a(i)%v+dt**2/(2*A%a(i)%m)*A%a(i)%F
  end do
end subroutine

subroutine Berechne_v(A,dt)
      type(Atome)::A; real::dt
  integer::i
  do i=1,A%N
    A%a(i)%v=A%a(i)%v+dt/(2*A%a(i)%m)*(A%a(i)%F+A%a(i)%Falt)
  end do 
end subroutine

subroutine Zeitintegration(A,dt,t_end)
      type(Atome)::A; real::dt,t_end
  integer::k
  integer::i
  real::t
  t=0
  k=0                                    ! Zaehler fuer Ausgabe
  call Berechne_F(A)
  do while(t<t_end)
    t=t+dt
    call Berechne_x(A,dt)
    do i=1,A%N
      A%a(i)%Falt=A%a(i)%F
    end do 
    call Berechne_F(A)
    call Berechne_v(A,dt)
    k=k+1
    if(mod(k,AUSGEBEN_ALLE)==0) call Ausgabe(A,t)
  end do
end subroutine

! ***  Startkonfiguration  ****************************************************

subroutine Startkonfiguration(A)
      type(Atome)::A
  ! Partikel initialisieren
  A%N=3
  allocate(A%a(A%N))
  ! Sonne:
  A%a(1)%typ=1; A%a(1)%m=318.0; A%a(1)%x=(/ 35.1,10.0,4.0/); A%a(1)%v=(/-0.1,-0.1,0.0/);
  ! Erde:
  A%a(2)%typ=2; A%a(2)%m=1.0; A%a(2)%x=(/ 1.0,-15.0,0.0/); A%a(2)%v=(/0.2,0.0,0.4/);
  !Jupiter:
  A%a(3)%typ=3; A%a(3)%m=318.0; A%a(3)%x=(/ 5.4,5.0,-2.0/); A%a(3)%v=(/0.1,0.1,0.0/);
  !Haley:
  !A%a(4)%typ=4; A%a(4)%m=3e-11; A%a(4)%x=(/ 35.1,10.0,0.0/); A%a(4)%v=(/0.0,0.0,0.0/);
end subroutine

end module

! ***  Hauptprogramm  *********************************************************

program Planeten

use Planeten_Modul

type(Atome)::A

open(unit=1,file="planeten.xyz",action="write",status="replace")

call Startkonfiguration(A)
call Ausgabe(A,0.0)

call Zeitintegration(A,DELTA_T,T_MAX)

close(unit=1)
deallocate(A%a)

end program 
