module Planeten_Modul

! Allgemeine Parameter
integer,parameter::DIM=2                 ! Raumdimension (2D oder 3D)
integer,parameter::AUSGEBEN_ALLE=8
integer,parameter::NX=25
integer,parameter::NY=15
integer,parameter::NXX=25
integer,parameter::NYY=15
integer,parameter::NACHBARN=3**DIM
integer,parameter::NGX=1.6*NX
integer,parameter::NGY=2*(NY+1.2*NYY)
real,parameter::   DELTA_T=0.01         ! Zeitschrittweite 
real,parameter::   T_MAX=40.0             ! Endzeit
real,parameter::   EPS=5.0
real,parameter::   SIGMA=1.0
real,parameter::   MASSE=1.0
real,parameter::   RGG=2**(1.0/6.0)*SIGMA
real,parameter::   TEMP=0.001             !lustig: 10, siedepunkt: 1.0
real,parameter::   GAMMA=1.177e-4        ! Gravitationskonstante GAMMA
real,parameter::   R_AB=2.5*SIGMA;
real,parameter::   FAKTOR=1.0
logical,parameter::ABSCHNEIDEN=.true.

! Eigene Datentypen
type Atom                                ! Himmelskoerper:
  real::m                                !   Masse
  real,dimension(DIM)::x,v,F,Falt        !   Ort,Geschwindigkeit,Kraft
  integer::typ,zelle                     !   Typ (fuer Darstellung)
end type
type Atome                               ! Sonnensystem:
  integer::N                             !   Anzahl der Himmelskoerper
  type(Atom),dimension(:),pointer::a     !   Feld der Himmelskoerper
end type
type Gebiet
  integer::N,Nx,Ny
  real::x0,y0
  real::d
  integer,dimension(:,:),pointer::Nachbar
  integer,dimension(:),pointer::Anzahl,Start
end type

contains

! ***  Zeitintegration und Ausgabe  *******************************************

subroutine Ausgabe(A,t)
      type(Atome)::A
      real::t
  integer::i
  write(unit=1,fmt=*) A%N
  write(unit=1,fmt=*) t," fs"
  do i=1,A%N
    write(unit=1,fmt=*) A%a(i)%typ, A%a(i)%x, 0, 0
  end do
end subroutine 

subroutine Kraft(A,i,j)
      type(Atome)::A; integer::i,j
  real,dimension(DIM)::xij
  real::r,rr,Fij
  integer::d
  xij=A%a(j)%x-A%a(i)%x                               ! Abstandsvektor von i nach j
  rr=0.0                                              ! r^2
  do d=1,DIM
    rr=rr+xij(d)**2
  end do
  r=sqrt(rr)                                         ! Abstand
  Fij=24*EPS/r*(SIGMA/r)**6*(1-2*(SIGMA/r)**6)     ! Kraft von j auf i
  A%a(i)%F=A%a(i)%F+Fij*xij/r
  A%a(j)%F=A%a(j)%F-Fij*xij/r
end subroutine

subroutine Berechne_F(G,A)
      type(Atome)::A
      type(Gebiet)::G
   integer::i,j,k,l
   do i=1,A%N
     A%a(i)%F=0
   end do
   do i=1,G%N
     do j=1,9
       if ((G%Nachbar(i,j)/=-1).and.(i<=G%Nachbar(i,j))) then
         do k=G%Start(i),G%Start(i)+G%Anzahl(i)-1
           do l=G%Start(G%Nachbar(i,j)),G%Start(G%Nachbar(i,j))+G%Anzahl(G%Nachbar(i,j))-1
             if (k<l) then
              call Kraft(A,k,l)
             end if
           end do
         end do
       end if
     end do
   end do
end subroutine

subroutine Berechne_x(A,dt)
      type(Atome)::A; real::dt
  integer::i
  do i=1,A%N
    A%a(i)%x=A%a(i)%x+dt*A%a(i)%v+dt**2/(2*A%a(i)%m)*A%a(i)%F
  end do
end subroutine

subroutine Berechne_v(A,dt)
      type(Atome)::A; real::dt
  integer::i
  do i=1,A%N
    A%a(i)%v=A%a(i)%v+dt/(2*A%a(i)%m)*(A%a(i)%F+A%a(i)%Falt)
  end do 
end subroutine

subroutine Zeitintegration(G,A,dt,t_end)
     type(Gebiet)::G;  type(Atome)::A; real::dt,t_end
  integer::k
  integer::i
  real::t
  t=0
  k=0
  call Berechne_F(G,A)
  do while(t<t_end)
    do i=1, A%N
      A%a(i)%zelle=Zellnummer(G,A,i)
    end do                                    ! Zaehler fuer Ausgabe
    t=t+dt
    call Berechne_x(A,dt)
    call Atome_sortieren(G,A)
    do i=1,A%N
      A%a(i)%Falt=A%a(i)%F
    end do 
    call Berechne_F(G,A)
    call Berechne_v(A,dt)
    k=k+1
    if(mod(k,AUSGEBEN_ALLE)==0) call Ausgabe(A,t)
  end do
end subroutine

! ***  Startkonfiguration  ****************************************************

subroutine Startkonfiguration(G,A)
      type(Atome)::A
      type(Gebiet)::G
  integer::i,j,k
  ! Partikel initialisieren
  A%N=(NX+1)*(NY+1)+(NYY+1)*(NXX+1)
  allocate(A%a(A%N))
  ! Sonne:
  !A%a(1)%typ=1; A%a(1)%m=333e3; A%a(1)%x=(/ 0.0,0.0,0.0/); A%a(1)%v=(/0.0,0.0,0.0/);
  ! Erde:
  !A%a(2)%typ=2; A%a(2)%m=1.0; A%a(2)%x=(/ 1.0,0.0,0.0/); A%a(2)%v=(/0.0,6.28,0.0/);
  !Jupiter:
  !A%a(3)%typ=3; A%a(3)%m=318.0; A%a(3)%x=(/ 5.4,0.0,0.0/); A%a(3)%v=(/0.0,2.86,0.0/);
  !Haley:
  !A%a(4)%typ=4; A%a(4)%m=3e-11; A%a(4)%x=(/ 35.1,0.0,0.0/); A%a(4)%v=(/0.0,0.192,0.0/);
  k=0
  do i=0,NX
    do j=0,NY
     k=k+1
     if(mod(j,2)==1) then
       A%a(k)%typ=(1); A%a(k)%m=MASSE; A%a(k)%x=(/(i-0.5*NX)*RGG,(j-0.5*NY)*RGG*(sqrt(3.0)/2)/); A%a(k)%v=(/0.0,2.0/)
     else
       A%a(k)%typ=(1); A%a(k)%m=MASSE; A%a(k)%x=(/(i-0.5*NX+0.5)*RGG,(j-0.5*NY)*RGG*(sqrt(3.0)/2)/); A%a(k)%v=(/0.0,2.0/)
    end if
    end do
  end do
  do i=0,NXX
    do j=0,NYY
     k=k+1
     if(mod(j,2)==1) then
       A%a(k)%typ=(2); A%a(k)%m=MASSE*FAKTOR; A%a(k)%x=(/(i-0.5*NXX)*RGG,(j+NY+NYY)*RGG*(sqrt(3.0)/2)/); A%a(k)%v=(/0.0,-2.0/)
     else
       A%a(k)%typ=(2); A%a(k)%m=MASSE*FAKTOR; A%a(k)%x=(/(i-0.5*NXX+0.5)*RGG,(j+NY+NYY)*RGG*(sqrt(3.0)/2)/)
       A%a(k)%v=(/0.0,-2.0/)
    end if

!     A%a(k)%typ=(2); A%a(k)%m=MASSE*FAKTOR; A%a(k)%x=(/(i-0.5*NXX)*RGG,(j+NY+NYY)*RGG/); A%a(k)%v=(/0.0,-6.0/)
    end do
  end do

call Temperatur(A,TEMP)
do i=1, A%N
  A%a(i)%zelle=Zellnummer(G,A,i)
end do  

end subroutine

function NV_Zufallszahl()
       real::NV_Zufallszahl
  real::a1,a2,r
  r=0
  do while ((r==0).or.(1<=r))
    call random_number(a1); a1=2*a1-1
    call random_number(a2); a2=2*a2-1
    r=a1**2+a2**2
  end do
  NV_Zufallszahl=a1*sqrt(-2*log(r)/r)
end function

subroutine Temperatur(A,T)
      type(Atome)::A; real::T
  integer::i,d
  real::s,z
  do i=1,A%N
    s=sqrt(T/A%a(i)%m)
    do d=1, DIM
      z=NV_Zufallszahl()*s
      A%a(i)%v(d)=A%a(i)%v(d)+z
    end do
  end do
end subroutine

! ***  Gebietsaufteilung  *****************************************************

function Zellnummer(G,A,i)
      integer::Zellnummer; type(Gebiet)::G; type(Atome)::A; integer::i
  integer::zx,zy
  zx=int((A%a(i)%x(1)-G%x0)/G%d)
  zy=int((A%a(i)%x(2)-G%y0)/G%d)
  if ( (0<=zx) .and. (zx<G%Nx) .and. (0<=zy) .and. (zy<G%Ny) ) then
    Zellnummer=1+zx+zy*G%Nx
  else
    Zellnummer=0
  end if
end function

function Zellen_benachbart(G,i,j)
      logical::Zellen_benachbart; type(Gebiet)::G; integer::i,j
  integer::ix,iy,jx,jy
  ix=mod((i-1),G%Nx); iy=int((i-1)/G%Nx)
  jx=mod((j-1),G%Nx); jy=int((j-1)/G%Nx)
  if ( (i/=0) .and. (j/=0) .and. (abs(ix-jx)<=1) .and. (abs(iy-jy)<=1) ) then
    Zellen_benachbart=.true.
  else
    Zellen_benachbart=.false.
  end if
end function

subroutine Atome_sortieren(G,A)
      type(Gebiet)::G; type(Atome)::A
  integer::i,k,l,z,zn
  type(Atom)::aa 
  ! Anzahl und Startposition fuer jedes Zelle bestimmen
  G%Anzahl(:)=0
  do k=1,A%N
    i=A%a(k)%zelle 
    G%Anzahl(i)=G%Anzahl(i)+1
  end do
  G%Start(0)=1
  do i=1,G%N
    G%Start(i)=G%Start(i-1)+G%Anzahl(i-1)
  end do
  ! Atome einsortieren
  G%Anzahl(:)=0    ! Zahl der richtig einsortierten Atome
  z=0              ! Aktuelle Zelle
  do while (z<G%N)      ! "<", da letzte Zelle automatisch richtig
    ! Solange noch Teilchen in Zelle z fehlen, durch Tauschen welche suchen
    do while ( G%Start(z)+G%Anzahl(z)<G%Start(z+1) )
      k=G%Start(z)+G%Anzahl(z)   ! erstes potentiell falsche Atom in Zelle z
      zn=A%a(k)%zelle            ! Zelle, wo das aktuelle Atom hin muss
      do while (zn/=z)
        ! aktuelles Atom richtig einsortieren und dortiges Atom herholen
        l=G%Start(zn)+G%Anzahl(zn)   ! hier muss das aktuelle Atom hin
        G%Anzahl(zn)=G%Anzahl(zn)+1  ! und hier ist es dann auch richtig
        aa=A%a(l); A%a(l)=A%a(k); A%a(k)=aa  ! Atome k und l tauschen
        zn=A%a(k)%zelle              ! Gebiet des geholten Atoms
      end do
      G%Anzahl(z)=G%Anzahl(z)+1  ! aktuelles Teilchen ist richtig einsortiert
    end do
    z=z+1               ! aktuelle Zelle fertig, also zur naechsten
  enddo
  ! letze Zelle wurde nicht ganz durchlaufen, deshalb noch Anzahl berechnen
  G%Anzahl(G%N)=A%N-G%Start(G%N)+1
end subroutine

subroutine Zellen_initialisieren(G)
      type(Gebiet)::G
  integer::i,j,k
  ! Zelleninitialisieren, Zelle 0 fr verlorene Teilchen
  G%x0=-0.5*NGX
  G%y0=-0.5*NGY
  G%d =R_AB
  G%Nx=int(NGX*RGG/G%d)+1
  G%Ny=int(NGY*RGG/G%d)+1
  G%N=G%Nx*G%Ny           ! Zahl der echten Zellen
  allocate (G%Anzahl(0:G%N))
  allocate (G%Start(0:G%N))
  allocate (G%Nachbar(0:G%N,NACHBARN))
  G%Nachbar(:,:)=-1       ! evtl. nicht vorhandene Nachbarn durch -1 markieren
  do i=0,G%N
    k=0                   ! Zahl der Nachbarzellen von Zelle i
    do j=0,G%N
      if(Zellen_benachbart(G,i,j)) then
        k=k+1
        G%Nachbar(i,k)=j  ! k-ter Nachbar von Zelle i ist j
      end if
    end do
  end do
end subroutine


end module

! ***  Hauptprogramm  *********************************************************

program Planeten

use Planeten_Modul

type(Atome)::A
type(Gebiet)::G

open(unit=1,file="stoss.xyz",action="write",status="replace")
call Zellen_initialisieren(G)
call Startkonfiguration(G,A)
call Atome_sortieren(G,A)
call Ausgabe(A,0.0)

call Zeitintegration(G,A,DELTA_T,T_MAX)

close(unit=1)
deallocate(A%a)
deallocate(G%Anzahl)
deallocate(G%Start)
deallocate(G%Nachbar)

end program 
